import re
from django.utils.safestring import mark_safe
from django.utils.html import escape
from html import unescape

__all__ = [
    'HASTAG_RE', 
    'replace_hashtags_with_hrefs', 
    'iter_hashtags',
    'iter_hashtags_at'
]

HASTAG_RE = re.compile(r'''[^&](\#([\'\"\w\-\_\d]+))''')
HASTAG_AT_RE = re.compile(r'''[^&](\@([\'\"\w\-\_\d]+))''')

HASTAG_RE_SEARCH = re.compile(r'''(\#([\'\"\w\-\_\d]+))''')
HASTAG_AT_RE_SEARCH = re.compile(r'''(\@([\'\"\w\-\_\d]+))''')

def replace_hashtags_with_hrefs(text, href='#{hastag}', label='#{hastag}'):
    text = unescape(text)
    href = href.format(hastag='\\2')
    label = label.format(hastag='\\2')
    html = HASTAG_RE.sub(f' <a href="{href}">{label}</a>', text)
    return mark_safe(html)


def iter_hashtags(text):
    if text:
        for full_hashtag, hashtag in HASTAG_RE_SEARCH.findall(text):
            yield hashtag

def iter_hashtags_at(text):
    if text:
        for full_hashtag, hashtag in HASTAG_AT_RE_SEARCH.findall(text):
            yield hashtag

def lazy_register(register):

    @register.filter(name='replace_hashtags_with_hrefs')
    def tt_replace_hashtags_with_hrefs(text, *args): return replace_hashtags_with_hrefs(text, *args)



import hashlib
import random
from django.utils.http import base36_to_int, int_to_base36

__all__ = ["random_token", "request_uid_token"]#, "base64_crypted_key", "base64_decrypted_key"

def random_token(extra=None, hash_func=hashlib.sha256):
    if extra is None:
        extra = []
    elif not isinstance(extra, list) or not isinstance(extra, tuple):
        extra = [str(extra)]
    bits = [ str(e) for e in extra ] + [str(random.SystemRandom().getrandbits(512))]
    return hash_func("".join(bits).encode('utf-8')).hexdigest()

# from Crypto.Cipher import AES
# def base64_crypted_key(string, passphrase=''):
#     try:
#         string = (( '%s|'% (string)) * 32)[0:32]
#         secret_key = passphrase.rjust(12)
#         cipher = AES.new(secret_key,AES.MODE_ECB)
#         return base64.b64encode(cipher.encrypt(string))
#     except:
#         return None


# def base64_decrypted_key(string, passphrase=''):
#     try:
#         secret_key = passphrase.rjust(12)
#         cipher = AES.new(secret_key,AES.MODE_ECB)
#         decoded = cipher.decrypt(base64.b64decode(string))
#         return decoded.strip().split('|')[0]
#     except:
#         return None


def request_uid_token(request):
    from django.contrib.auth.tokens import default_token_generator
    uid = int_to_base36(request.user.id)
    token = default_token_generator.make_token(request.user)
    return uid, token





"""
Django-Select2 URL configuration.
Add `django_select` to your ``urlconf`` **if** you use any 'Model' fields::
    url(r'^internal/', include('workon.urls')),
"""
from django.conf.urls import url

from workon.views.select2 import AutoResponseView

urlpatterns = [
    url(r"^fields/auto.json$", AutoResponseView.as_view(), name="django_select2-json"),
]


import re, os, functools
import sys
from urllib.parse import urlparse, urlunparse
from django.conf.urls import url
from django.apps import apps
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.urls import reverse, exceptions, NoReverseMatch
from django.http.request import HttpRequest
from django.core.exceptions import SuspiciousOperation
from workon.utils.cache import memoize

_url_composite = r"""(?i)\b((?:https?:(?:/{1,3}|[a-z0-9%])|[a-z0-9.\-]+[.](?:com|net|org|edu|gov|mil|aero|asia|biz|cat|coop|info|int|jobs|mobi|museum|name|post|pro|tel|travel|xxx|ac|ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cs|cu|cv|cx|cy|cz|dd|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|Ja|sk|sl|sm|sn|so|sr|ss|st|su|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tp|tr|tt|tv|tw|tz|ua|ug|uk|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|yu|za|zm|zw)/)(?:[^\s()<>{}\[\]]+|\([^\s()]*?\([^\s()]+\)[^\s()]*?\)|\([^\s]+?\))+(?:\([^\s()]*?\([^\s()]+\)[^\s()]*?\)|\([^\s]+?\)|[^\s`!()\[\]{};:'".,<>?«»“”‘’])|(?:(?<!@)[a-z0-9]+(?:[.\-][a-z0-9]+)*[.](?:com|net|org|edu|gov|mil|aero|asia|biz|cat|coop|info|int|jobs|mobi|museum|name|post|pro|tel|travel|xxx|ac|ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cs|cu|cv|cx|cy|cz|dd|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|Ja|sk|sl|sm|sn|so|sr|ss|st|su|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tp|tr|tt|tv|tw|tz|ua|ug|uk|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|yu|za|zm|zw)\b/?(?!@)))"""
_url_regex = re.compile(_url_composite)
_url_regex_multiline = re.compile(_url_composite, re.MULTILINE|re.UNICODE)

__all__ = [
    'append_protocol',
    'extract_urls',
    'urls_to_html',
    'extract_urls_to_html',
    'replace_urls_to_href',
    'get_current_site_domain',
    'build_absolute_url',
    'get_current_site',
    'external_url',
    'canonical_url',
    'canonical_url_static',
    'absolute_url',
    'url_signature',
    'default_redirect',
    'ensure_safe_url',
]



def append_protocol(url):
    if url:
        if not (url.startswith('http://') or url.startswith('https://')):
            url = f"http://{url}"
    return url

def extract_urls(text):
    if text is not None:
        urls = []
        for url in _url_regex.findall(text):
            if not url.startswith('//'):
                urls.append(append_protocol(url))

        return list(set(urls))
    else:
        return []

def urls_to_html(urls, reverse=True, target="_blank", hide_protocol=True, classname=None, divider="<br />"):

    urls = [
        u'<a %shref="%s" %s/>%s</a>' % (
            ('target="%s" ' % target) if target else "",
            url,
            ('class="%s" ' % classname) if classname else "",
            (url.replace('https://', '').replace('http://', '') if hide_protocol else url).strip('/')
        ) for url in urls
    ]
    if reverse:
        urls.reverse()
    html = divider.join(urls)
    return html

def extract_urls_to_html(text, **kwargs):
    return urls_to_html(extract_urls(text), **kwargs)


def replace_urls_to_href(text, target="_blank", hide_protocol=True):

    text = _url_regex_multiline.sub(r'<a href="http://\1" %s rel="nofollow">\1</a>' % (
        ('target="%s" ' % target) if target else ""
    ), text)
    text = text.replace('http://http', 'http')
    # Replace email to mailto
    # urls = re.compile(r"([\w\-\.]+@(\w[\w\-]+\.)+[\w\-]+)", re.MULTILINE|re.UNICODE)
    # value = urls.sub(r'<a href="mailto:\1">\1</a>', value)
    return text

# urlfinder = re.compile("([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}|((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+):[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\),\\\"]")

# def urlify2(value):
#     return urlfinder.sub(r'<a href="\1">\1</a>', value)

def get_current_site_domain(request=None):
    try:
        from django.contrib.sites.models import Site
        domain = Site.objects.get_current().domain
    except:
        domain = getattr(settings, 'APP_DOMAIN', '')
    return domain

def build_absolute_url(url="", request=None):
    domain = get_current_site_domain(request=request)
    return "{0}://{1}{2}".format(
        getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http"),
        domain.split('//')[-1],
        url
    ).strip('/')


def external_url(url):
    if not url.startswith('http') and not url.startswith('//'):
        return f"http://{url}"
    return url


def canonical_url(url, domain_check=False):
    """
    Ensure that the url contains the `http://mysite.com` part,
    particularly for requests made on the local dev server
    """

    domain = get_current_site_domain()
    if not domain.startswith('http') and not domain.startswith('//'):
        domain = f"http://{domain}"

    if not url.startswith('http') and not url.startswith('//'):
        url = os.path.join(domain, url.lstrip('/'))

    if domain_check:
        url_parts = URL(url)
        current_site_parts = URL(URL().domain(domain).as_string())
        if url_parts.subdomains()[-2:] != current_site_parts.subdomains()[-2:]:
            raise ValueError("Suspicious domain '%s' that differs from the "
                "current Site one '%s'" % (url_parts.domain(), current_site_parts.domain()))

    return url

def absolute_url(*args, **kwargs):
    return canonical_url(*args, **kwargs)

def canonical_url_static(url, domain_check=False):# False because of S3
    """
    Ensure that the url contains the `http://mysite.com/STATIC_URL` part,
    particularly for requests made on the local dev server
    """
    if url.startswith('http') or url.startswith('//'):
        return url
    return canonical_url( os.path.join(settings.STATIC_URL, url), domain_check)


def url_signature(resolver_match):
    """
    Convert
        a `django.core.urlresolvers.ResolverMatch` instance
        usually retrieved from a `django.core.urlresolvers.resolve` call
    To
        'namespace:view_name'

    that `django.core.urlresolvers.reverse` can use
    """
    signature = resolver_match.url_name
    if resolver_match.namespace:
        signature = "%s:%s" % (resolver_match.namespace, signature)
    return signature


def default_redirect(request, fallback_url, **kwargs):
    redirect_field_name = kwargs.get("redirect_field_name", "next")
    next_url = request.POST.get(redirect_field_name, request.GET.get(redirect_field_name))
    if not next_url:
        # try the session if available
        if hasattr(request, "session"):
            session_key_value = kwargs.get("session_key_value", "redirect_to")
            if session_key_value in request.session:
                next_url = request.session[session_key_value]
                del request.session[session_key_value]
    is_safe = functools.partial(
        ensure_safe_url,
        allowed_protocols=kwargs.get("allowed_protocols"),
        allowed_host=request.get_host()
    )
    if next_url and is_safe(next_url):
        return next_url
    else:
        try:
            fallback_url = reverse(fallback_url)
        except NoReverseMatch:
            if callable(fallback_url):
                raise
            if "/" not in fallback_url and "." not in fallback_url:
                raise
        # assert the fallback URL is safe to return to caller. if it is
        # determined unsafe then raise an exception as the fallback value comes
        # from the a source the developer choose.
        is_safe(fallback_url, raise_on_fail=True)
        return fallback_url


def ensure_safe_url(url, allowed_protocols=None, allowed_host=None, raise_on_fail=False):
    if allowed_protocols is None:
        allowed_protocols = ["http", "https"]
    parsed = urlparse(url)
    # perform security checks to ensure no malicious intent
    # (i.e., an XSS attack with a data URL)
    safe = True
    if parsed.scheme and parsed.scheme not in allowed_protocols:
        if raise_on_fail:
            raise SuspiciousOperation("Unsafe redirect to URL with protocol '{0}'".format(parsed.scheme))
        safe = False
    if allowed_host and parsed.netloc and parsed.netloc != allowed_host:
        if raise_on_fail:
            raise SuspiciousOperation("Unsafe redirect to URL not matching host '{0}'".format(allowed_host))
        safe = False
    return safe



from functools import partial

__all__ = ["memoize"]

class memoize(object):
    """Decorator that caches a function's return value each time it is called.
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.
    """
    def __init__(self, func):
        self.func = func
        self.cache = {}
    def __call__(self, *args):
        try:
            return self.cache[args]
        except KeyError:
            value = self.func(*args)
            self.cache[args] = value
            return value
        except TypeError:
             # uncachable -- for instance, passing a list as an argument.
             # Better to not cache than to blow up entirely.
            return self.func(*args)
    def __repr__(self):
        """Return the function's docstring."""
        return self.func.__doc__
    def __get__(self, obj, objtype):
        """Support instance methods."""
        fn = partial(self.__call__, obj)
        fn.reset = self._reset
        return fn
    def _reset(self):
        self.cache = {}


import os, uuid, datetime

from django.utils.encoding import force_str, force_text
from django.utils.deconstruct import deconstructible

__all__ = ["UniqueFilename", "unique_filename"]

@deconstructible
class UniqueFilename(object):
    path = "example/{0}/{1}{2}"

    def __init__(self, sub_path, original_filename_field=None):
        self.sub_path = sub_path
        self.original_filename_field = original_filename_field

    def __call__(self, instance, filename):
        if self.original_filename_field and hasattr(instance, self.original_filename_field):
            setattr(instance, self.original_filename_field, filename)
        parts = filename.split('.')
        extension = parts[-1]
        directory_path = os.path.normpath(force_text(datetime.datetime.now().strftime(force_str(self.sub_path))))
        unique_name = u"{0}.{1}".format(uuid.uuid4(), extension)
        return os.path.join(directory_path, unique_name)

# retro compatibility for older uses as function
class unique_filename(UniqueFilename):
    pass


import datetime
from django import forms
from django.db import models
from django.views import generic
from django.shortcuts import render
from django.utils.html import mark_safe
from django.utils import six
from django.utils.text import slugify
from django.forms.fields import CallableChoiceIterator
from django.core.exceptions import FieldDoesNotExist
import workon.utils
import workon.conf


__all__ = ['ListColumn', 'ListCol', 'ListFilter', 'ListAction', 'List', 'TableList']


    









class ModelListChoiceField(forms.ModelChoiceField):

    empty_label= ''

    pass

class ListChoiceField(forms.ChoiceField):
    def _get_choices(self):
        return self._choices

    def _set_choices(self, value):
        if callable(value):
            value = CallableChoiceIterator(value)
        else:
            value = list(value)
        self._choices = self.widget.choices = [('', ' -- '), ] + list(value)
    choices = property(_get_choices, _set_choices)


class ListMultipleChoiceField(forms.MultipleChoiceField):
    def _get_choices(self):
        return self._choices

    def _set_choices(self, value):
        if callable(value):
            value = CallableChoiceIterator(value)
        else:
            value = list(value)
        self._choices = self.widget.choices = list(value)
    choices = property(_get_choices, _set_choices)
















class ListAction():
    def __init__(self, name, label=None, col=2):
        self.name = name
        self.meta = {
            'class': f'col s12 m{col}',
            'label': label or name,
        }










class ListFilter():
    def __init__(self, name, label=None, col=2, choices=None, classes='', multiple=False, default=None, field_class=None, field_kwargs=None):
        self.name = name
        field_kwargs = field_kwargs if field_kwargs else {}
        field_kwargs['required'] = False
        field_kwargs['initial'] = default
        field_kwargs['label'] = label
        if not field_class:
            if choices is not None:
                if isinstance(choices, models.QuerySet):
                    field_class = ModelListChoiceField
                    field_kwargs['queryset'] = choices
                    field_kwargs['empty_label'] = ''
                else:
                    if multiple == True:
                        field_class = ListMultipleChoiceField
                    else:
                        field_class = ListChoiceField
                    field_kwargs['choices'] = choices
            else:
                field_class = forms.CharField
        self.form_field_kwargs = field_kwargs
        self.form_field_class = field_class
        self.meta = {
            'class': f'{classes}',
            'label': label or name,
        }



class ListForm(forms.Form):
    def __init__(self, filters_instances,  *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['page'] = forms.CharField(widget=forms.HiddenInput(), required=False)
        for fi in filters_instances:
            field_kwargs = fi.form_field_kwargs
            self.fields[fi.name] = fi.form_field_class(**field_kwargs)
            self.fields[fi.name].meta = fi.meta





class ListColumn():
    def __init__(self, name, label=None, classes='', attrs='', ellipsis=False, col=None):
        self.name = name
        self.label = label
        self.classes = classes
        self.attrs = attrs
        self.ellipsis = ellipsis

    def __str__(self):
        return str(self.label) if self.label else ''

class ListCol(ListColumn): pass






class List(generic.FormView):

    template_name = workon.conf.LIST_TEMPLATE
    results_template_name = workon.conf.LIST_RESULTS_TEMPLATE
    row_template_name = workon.conf.LIST_ROW_TEMPLATE
    layout_template_name = workon.conf.LIST_LAYOUT_TEMPLATE

    list_id = 'filtered_list'
    list_class = ''
    filters = []
    actions = []
    columns = []
    filters_instances = []
    columns_instances = []
    form_class = None
    model = None

    def make_column_instance(self, col, **kwargs):
        if isinstance(col, ListCol):
            col = col

        elif isinstance(col, dict):
            col = ListCol(**col)

        elif isinstance(col, tuple):
            col = self.make_column_instance(col[0], label=col[-1])

        elif isinstance(col, six.string_types):
            if self.model and not 'label' in kwargs:
                try:
                    kwargs['label'] = self.model._meta.get_field(col).verbose_name
                except FieldDoesNotExist:
                    pass
            col = ListCol(col, **kwargs)
        return col

    def make_filter_instance(self, fi, **kwargs):
        if isinstance(fi, ListFilter):
            fi = fi

        elif isinstance(fi, dict):
            fi = ListFilter(**fi)

        elif isinstance(fi, tuple):
            fi = self.make_filter_instance(fi[0], label=fi[-1])

        elif isinstance(fi, six.string_types):
            ftype_name = fi.split(':', 1)
            ftype = ftype_name[0]
            kwargs['label'] = name = ftype_name[-1]
            if len(ftype_name) == 2:
                if ftype == "int":
                    kwargs['field_class'] = forms.IntegerField
                elif ftype == "bool":
                    kwargs['field_class'] = forms.BooleanField
            fi = ListFilter(name, **kwargs)
        return fi

    def render_cell(self, column_instance, obj):
        data = dict(
            attrs='',
            classes='',
            column=column_instance
        )
        name = column_instance.name 
        value_method = getattr(self, f'get_col_{name}_value', None)
        if value_method:
            value = value_method(obj)
        else:
            value = getattr(obj, name, None)
            if column_instance.ellipsis:
                data['attrs'] += f'data-tooltip="{value}"'
        if value is None or value is "":
            value = f'''<i class="icon disabled">block</i>'''
        elif isinstance(value, bool):
            value = f'''<i class="icon {'success' if value else 'error'}">{'check' if value else 'close'}</i>'''
        elif isinstance(value, datetime.date):
            value = value.strftime("%d %b <sup>%Y</sup> <sub>à %H:%M</sub>").replace('<sub>à 00:00</sub>', '')
        elif isinstance(value, six.string_types):
            value = value
        data['value'] = value 

        class_method = getattr(self, f'get_col_{name}_class', None)
        if class_method:
            data['classes'] += class_method(obj)

        attrs_method = getattr(self, f'get_col_{name}_attrs', None)
        if attrs_method:
            data['attrs'] += attrs_method(obj)

        # data['attrs'] = " ".join([f'{name}="{value}"' for name, value in data['attrs'].items()]) 
        return data


    def get_form(self):

        self.columns_instances = [ self.make_column_instance(fi) for fi in getattr(self, 'fields', self.columns)]
        self.filters_instances = [ self.make_filter_instance(fi) for fi in self.filters]

        if not self.form_class:
            self.form = ListForm(self.filters_instances, **self.get_form_kwargs())
        else:
            self.form = self.form_class(**self.get_form_kwargs())
        self.form_initialize(self.form)
        return self.form

    def form_initialize(self, form):
        return form
        
    def get_form_kwargs(self):
        kwargs = {
            'initial': self.get_initial(),
            'prefix': self.get_prefix(),
        }
        if self.request.method in ('GET'):
            kwargs.update({
                'initial': self.request.GET,
            })

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return kwargs

    def get_extra_actions(self, obj):
        return ''

    def get_row_id(self, obj):
        if hasattr(obj, 'pk'):
            return f'{obj}_{obj.pk}'
        else:
            return str(obj)

    def get_row_class(self, obj):
        return ''
        
    def get_row_style(self, obj):
        return ''

    def get_row_attrs(self, obj):
        if workon.conf.LIST_ROW_UPDATE_ON_DOUBLE_CLICK:
            update_url = self.get_row_update_url(obj)
            if update_url:
                return f'''\
                            {self.get_row_update_on_double_click_method(obj)}="{update_url}" \
                            data-tooltip='{{ 
                                "content": "Double-clic pour éditer" ,
                                "position": "left" 
                            }}' '''
        return ''

    def get_row_create_url(self, obj):
        return None

    def get_row_view_method(self, obj):
        return workon.conf.LIST_ROW_VIEW_METHOD

    def get_row_view_url(self, obj):
        return getattr(obj, 'view_url', lambda: None)()

    def get_row_update_method(self, obj):
        return workon.conf.LIST_ROW_UPDATE_METHOD

    def get_row_update_url(self, obj):
        return getattr(obj, 'update_url', lambda: None)()

    def get_row_update_on_double_click_method(self, obj):
        return workon.conf.LIST_ROW_UPDATE_ON_DOUBLE_CLICK_METHOD

    def get_row_delete_method(self, obj):
        return workon.conf.LIST_ROW_DELETE_METHOD

    def get_row_delete_url(self, obj):
        return getattr(obj, 'delete_url', lambda: None)()


    def get_template_names(self):
        return self.results_template_name if self.request.is_ajax() else self.template_name

    def get_queryset(self):
        qs = []
        if hasattr(self, 'model'):
            qs = self.model.objects.all()
        return self.filter(qs, self.F)

    def filter(self, qs, filters=None):
        if hasattr(self, 'model'):
            pass
        return qs 

    def get_row_update_method(self, obj):
        return ""


    def get_rows(self):
        self.rows = []
        self.queryset = getattr(self, 'queryset', [])
        # data = getattr(self.form, 'cleaned_data', { f.name: '' for f in self.filters_instances })
        for obj in self.queryset:
            cells = []
            for column_instance in self.columns_instances:
                cells.append(self.render_cell(column_instance, obj))

            row = {
                'object': obj,
                'id': self.get_row_id(obj),
                'attrs': self.get_row_attrs(obj),
                'class': self.get_row_class(obj),
                'style': self.get_row_style(obj),
                'cells': cells,
                'update_method': self.get_row_update_method(obj),
                'delete_method': self.get_row_delete_method(obj),
                'view_method': self.get_row_view_method(obj),
                'extra_actions': self.get_extra_actions(obj),
            }
            self.rows.append(row)
        return self.rows

    def get_breadbrumbs(self):
        return []


    def form_valid(self, form):
        self.data = self.F = form.cleaned_data
        qs = self.get_queryset()
        action = self.request.GET.get('_action')
        if action and hasattr(self, f'action_{action}'):
            response = getattr(self, f'action_{action}')(qs)
            if not response:
                qs = self.get_queryset()
            else:
                return response
        # try:
        self.queryset = workon.utils.DiggPaginator(qs, 50, body=6, padding=2).get_queryset_for_page(self.data.get('page'))
        # except:
        #     self.queryset = qs
        print(self.kwargs)
        return render(self.request, self.get_template_names(), self.get_context_data())

    def form_invalid(self, form):
        return self.form_valid(form)

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)

        create_method = getattr(self, 'get_create_method', None)
        ctx['create_method'] = create_method() if create_method else None
        create_url = getattr(self, 'get_create_url', None)
        ctx['create_url'] = create_url() if create_url else None 

        ctx['breadcrumbs'] = self.get_breadbrumbs()
        ctx['columns'] = self.columns_instances
        ctx['actions'] = self.actions
        ctx['rows'] = self.get_rows()
        ctx['list_results_template'] = self.results_template_name
        ctx['list_row_template'] = self.row_template_name
        ctx['layout_template'] = self.layout_template_name
        ctx['list_id'] = self.list_id
        ctx['list_class'] = self.list_class
        ctx['queryset'] = getattr(self, 'queryset', [])

        return ctx

class RowList(List):
    template_name = "workon/views/list/list.html"
    results_template_name = "workon/views/list/_list.html"
    row_template_name = "workon/views/list/_row.html"
    
    row_nums = 12
    row_tag = 'span'

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx['row_nums'] = self.row_nums
        ctx['row_tag'] = self.row_tag
        return ctx


class TableList(List):

    template_name = "workon/views/list/list.html"
    results_template_name = "workon/views/list/_list.html"
    row_template_name = "workon/views/list/_row.html"


from django.apps import AppConfig
from django.conf import settings
import workon.utils


class WorkonConfig(AppConfig):

    name = 'workon'

WORKON = getattr(settings, 'WORKON', dict())

LIST_LAYOUT_TEMPLATE = WORKON.get('LIST_LAYOUT_TEMPLATE', 'workon/views/list/layout.html')
LIST_TEMPLATE = WORKON.get('LIST_TEMPLATE', 'workon/views/list/list.html')
LIST_RESULTS_TEMPLATE = WORKON.get('LIST_RESULTS_TEMPLATE', 'workon/views/list/_list.html')
LIST_ROW_TEMPLATE = WORKON.get('LIST_ROW_TEMPLATE', 'workon/views/list/_row.html')
LIST_DEFAULT_METHOD = WORKON.get('LIST_DEFAULT_METHOD', 'data-modal')
LIST_ROW_CREATE_METHOD = WORKON.get('LIST_ROW_CREATE_METHOD', LIST_DEFAULT_METHOD)
LIST_ROW_VIEW_METHOD = WORKON.get('LIST_ROW_VIEW_METHOD', LIST_DEFAULT_METHOD)
LIST_ROW_UPDATE_METHOD = WORKON.get('LIST_ROW_UPDATE_METHOD', LIST_DEFAULT_METHOD)
LIST_ROW_UPDATE_ON_DOUBLE_CLICK = WORKON.get('LIST_ROW_UPDATE_ON_DOUBLE_CLICK', True)
LIST_ROW_UPDATE_ON_DOUBLE_CLICK_METHOD = WORKON.get('LIST_ROW_UPDATE_ON_DOUBLE_CLICK_METHOD', 'data-dblclick-modal')
LIST_ROW_DELETE_METHOD = WORKON.get('LIST_ROW_DELETE_METHOD', LIST_DEFAULT_METHOD)


SELECT2_CACHE_BACKEND = WORKON.get('SELECT2_CACHE_BACKEND','default')
SELECT2_CACHE_PREFIX = WORKON.get('SELECT2_CACHE_PREFIX','select2_')


from django.conf import settings
from django import forms
from django.utils.safestring import mark_safe
from django.contrib.admin import widgets as admin_widgets
from django.forms.utils import flatatt
from django.utils.html import strip_tags
from django.utils.html import escape
from django.template import Context
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.translation import get_language, ugettext as _
import json
try:
    from django.utils.encoding import smart_text as smart_unicode
except ImportError:
    try:
        from django.utils.encoding import smart_unicode
    except ImportError:
        from django.forms.util import smart_unicode


class CodeField(forms.CharField):
    def __init__(self, *args, **kwargs):
        super(CodeField, self).__init__(*args, **kwargs)


class CodeInput(forms.Textarea):


    def __init__(self, *args, **kwargs):
        self.mode = kwargs.get('attrs', {}).get('mode', 'python')
        super().__init__(*args, **kwargs)

    def render_script(self, id):
        return u'''
                <div id="%(id)s_ace_editor"></div>
                <script type="text/javascript">
                    var %(id)s_ed = ace.edit("%(id)s_ace_editor");
                    %(id)s_ed.setTheme("ace/theme/monokai");
                    %(id)s_ed.getSession().setMode("ace/mode/%(mode)s");
                    //%(id)s_ed.setPrintMarginColumn(150)
                    %(id)s_ed.setOptions({
                        maxLines: Infinity
                    });
                    %(id)s_ed.on("change", function(e) {
                        $('#%(id)s').val(%(id)s_ed.getValue());
                    });
                    %(id)s_ed.setValue($('#%(id)s').val());
                    %(id)s_ed.resize();

                </script>
                ''' % { 'id' : id.replace('-', '_'), 'mode': self.mode }


    def render(self, name, value, attrs={}):
        if 'id' not in attrs:
            attrs['id'] = "id_%s" % name
        attrs['style'] = "display:none;"
        render = super().render(name, value, attrs)
        return mark_safe("%s%s" % (render, self.render_script(attrs['id'])))


# encoding: utf-8
import re
import os
import logging
import json

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db.models import CharField
from django.forms.fields import RegexField
from django.template.loader import render_to_string
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe

logger = logging.getLogger(__name__)

class ColorField(forms.CharField):

    def __init__(self, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = ColorInput(attrs={'placeholder': '#xxxxxx'})
        super(ColorField, self).__init__(*args, **kwargs)


class ColorInput(forms.widgets.TextInput):

    class Media:
        css = {
            'all': (settings.STATIC_URL + 'workon/css/colpick/colpick.css',)
        }
        js = (settings.STATIC_URL + 'workon/js/colorpicker.js',)

    input_type = 'text'

    def __init__(self, *args, **kwargs):

        options = kwargs.pop('options', {})
        super(ColorInput, self).__init__(*args, **kwargs)

        attrs = {
            'data-colorpicker-input': json.dumps(options),
            #'style': 'border-right: 30px solid black'
        }
        self.attrs.update(attrs)

    def render_script(self, id):
        return ""
        # return '''<script type="text/javascript">

        #             $('#%(id)s').on('mousedown', function() {
        #                 if(!this.colpick) {
        #                     $(this).colpick({
        #                         layout: 'hex',
        #                         submit: 0,
        #                         livePreview: false,
        #                         onChange: function(hsb,hex,rgb,el,bySetColor) {
        #                             $(el).css('border-right-color','#'+hex);
        #                             if(!bySetColor) $(el).val('#'+hex);
        #                             $(el).trigger('change')
        #                         }
        #                     }).keyup(function(){
        #                         $(this).colpickSetColor(this.value.replace('#', ''));
        #                     });
        #                     this.colpick = true;
        #                 }
        #             }).on('mouseup', function() {
        #                 $(this).trigger('change')
        #             });
        #             $('#%(id)s').bind('inview', function() {
        #                 $(this).css('border-right-color',$(this).val());
        #             }).css('border-right-color',$('#%(id)s').val());
        #         </script>
        #         ''' % { 'id' : id }


    def render(self, name, value, attrs={}):
        if 'id' not in attrs:
            attrs['id'] = "id_%s" % name
        attrs['style'] = 'border-right: 30px solid %s' % (value if value else '#000')
        render = super(ColorInput, self).render(name, value, attrs)
        return mark_safe("%s%s" % (render, self.render_script(attrs['id'])))








RGB_REGEX = re.compile('^#?((?:[0-F]{3}){1,2})$', re.IGNORECASE)
class RGBColorField(CharField):
    widget = ColorInput
    default_validators = [RegexValidator(regex=RGB_REGEX)]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 7
        super(RGBColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.update({
            'form_class': RegexField,
            'widget': self.widget,
            'regex': RGB_REGEX
        })
        return super(RGBColorField, self).formfield(**kwargs)

    def south_field_triple(self):
        return 'libs.forms.fields.color.RGBColorField', [], {}

    def deconstruct(self):
        name, path, args, kwargs = super(RGBColorField, self).deconstruct()
        del kwargs['max_length']
        return name, path, args, kwargs


try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^libs\.forms\.fields\.color\.RGBColorField"])
except ImportError:
    pass



from django.conf import settings
from django.contrib import auth
from django.contrib.auth import get_user_model
import workon.utils

__all__ = [
    "get_user_or_none",
    "get_or_create_user"
]

def get_user_or_none(email):
    email = workon.utils.is_valid_email(email)
    if email:
        try:
            user = User.objects.get(email__iexact=email)
        except User.DoesNotExist:
            return None
    return None


def get_or_create_user(expiration_date=None, set_names_from_email=False,
                        password=None, save=True, **kwargs):
    User = get_user_model()
    attr_name = User.USERNAME_FIELD
    unique_field = kwargs.get(attr_name, None)
    if attr_name == "email":
        unique_field = workon.utils.is_valid_email(unique_field)
    if unique_field:
        try:
            user = User.objects.get(**{f'{attr_name}__iexact': unique_field })
            user._created = False
        except User.DoesNotExist:
            kwargs[attr_name] = unique_field
            user = User(
                **kwargs
                # expiration_date = expiration_date
            )
            if password:
                user.set_password(password)
            else:
                user.set_unusable_password()
            if set_names_from_email:

                if attr_name == "email" and not user.username:
                    user.username = unique_field.split('@')[0][0:254]

                if not user.first_name and not user.last_name:
                    user_names = user.username.split('.', 2)
                    if len(user_names) == 2:
                        user.first_name = ''.join([i for i in user_names[0].capitalize() if not i.isdigit()])
                        user.last_name = ''.join([i for i in user_names[1].capitalize() if not i.isdigit()])
                    else:
                        user.first_name = ''.join([i for i in user_names[0].capitalize() if not i.isdigit()])
            if save:
                user.save()
            user._created = True
        return user
    else:
        return None


import re
import os
import logging
import locale
import json
import datetime, time
from django import forms
from django.conf import settings
from django.db.models import CharField
from django.core.exceptions import ValidationError
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe

logger = logging.getLogger(__name__)

import workon.utils

class IconField(forms.CharField):

    def __init__(self, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = IconInput(attrs={'placeholder': '#xxxxxx'})
        super().__init__(*args, **kwargs)



class IconInput(forms.widgets.TextInput):

    class Media:
        js = ['contrib/forms/icon.js']
        css = {
            'all': (
                # settings.STATIC_URL + 'fonts/icomoon/style.css',
                # 'https://maxcdn.bootstrapcdn.com/font-awesome/4.3.0/css/font-awesome.min.css',
            )
        }

    # currency = DEFAULT_CURRENCY

    def __init__(self, *args, **kwargs):

        self.styles = kwargs.pop('styles')
        # if 'currency' in kwargs:
        #     self.currency = kwargs['currency']
        #     del kwargs['currency']
        super(IconInput, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        root = workon.utils.get_project_root()

        id = str(time.time()).replace('.', '')

        html = """<style>

            .contrib-icon-widget {
                color:rgb(102, 71, 30); margin:5px; text-decoration: none; cursor: pointer;   display: inline-block;
                font-size: 21px;
                transition: all 0.4s ease-out;
            }
            .contrib-icon-widget:hover {
                text-decoration: none; color:rgb(30, 31, 102);   display: inline-block;
                transition: all 0.4s ease-out;
            }
            .contrib-icon-widget.active,
            .contrib-icon-widget.active i,
            .contrib-icon-widget.active a,
            .contrib-icon-widget.active span {
                color:green; font-size: 25px; font-weight: bold;
            }

        </style>
        <div id="icons-%s">
            <div style="" class="contrib-icon-widget-tabs">""" % id

        for title, local_path, path, classname, prefix in self.styles:
            html += """<link href="%s" rel="stylesheet">""" % (path)

        tabs = "<ul>"
        contents = ""

        i = 1
        for title, local_path, path, classname, prefix in self.styles:
            filec = open(os.path.join(root, local_path), 'rb').read()
            filec = filec.replace(r'\s','')

            # title = re.search(r"@font-face\{.+font-family:([\'\"\d\w\-\s]+)\;", filec)
            # title = title.group(1) if title else "Icons"
            # title = title.strip().strip("'").strip('"').strip()

            tabs += """<li><a class="contrib-icon-widget-tabs-control" href="#tab-%s-%s">%s</a></li>""" % (id, i, title)
            contents += """<div class="contrib-icon-widget-tabs-content" id="tab-%s-%s" style="display:none;">"""  % (id, i)
            for item in re.finditer(
                r"\.(%s[\w\-\.]+)\:before" % classname,
                filec
            ):
                classname = item.group(1)
                classname = prefix + ' ' + classname
                contents += """<a class="contrib-icon-widget %s" data-value="%s" data-input="#%s"><i class="%s"></i></a>""" % (
                    "active" if value == classname else "",
                    classname,
                    attrs['id'],
                    classname
                )
            contents += "</div>"
            i += 1

        tabs += "</ul>"
        html += tabs + contents

        html += """</div>
                %(inherit)s
            </div>
        """ % {
            'inherit' : super(IconInput, self).render(name, value, attrs=attrs),
            'id': id
        }
        return mark_safe(html)



import re
import os
import logging
from django.conf import settings
from django import forms as base_forms
from django.template import Context
from django.forms.widgets import ClearableFileInput
from django.utils.encoding import force_text
from django.template.loader import render_to_string
from django.forms.utils import flatatt

try:
    from sorl.thumbnail.shortcuts import get_thumbnail
except:
    def get_thumbnail(image_url, *args, **kwargs):
        return image_url

logger = logging.getLogger(__name__)

class ImageInput(ClearableFileInput):
    # template_name = 'workon/forms/clearable_file_input.html'
    attrs = {'accept': 'image/*'}


    # def get_context(self, name, value, attrs):

    #     context = super().get_context(name, value, attrs)
    #     image_url = self.format_value(value)
    #     image_original_url = None
    #     image_thumb = None
    #     if image_url:
    #         image_original_url = os.path.join(settings.MEDIA_URL, image_url.name)
    #         try:
    #             image_thumb = get_thumbnail(image_url, 'x150', crop='center', upscale=True, format='PNG')
    #         except IOError as inst:
    #             logger.error(inst)
    #     context.update({
    #         'image_thumb': image_thumb,
    #         'image_url': image_url,
    #         'image_original_url': image_original_url,
    #         'image_id': f'{attrs["id"]}-image',
    #         'name': f'{name}',
    #     })
    #     return context



# unsable ?????????????????
class ImageField(base_forms.ImageField):
    widget = ClearableFileInput
    def __init__(self, *args, **kwargs):

        widget = ImageInput
        kwargs['widget'] = widget

        super(ImageField, self).__init__(*args, **kwargs)


class CroppedImageInput(base_forms.widgets.TextInput):
    class Media:
        css = {
            'all': (
                settings.STATIC_URL + 'workon/forms/image_cropped.css',
            )
        }
        js = (
            settings.STATIC_URL + 'workon/forms/image_cropped.js',
            # settings.STATIC_URL + 'js/cropper.js',
        )

class CroppedImageField(base_forms.CharField):
    widget = CroppedImageInput
    default_error_messages = {
        'invalid_image': 'Upload a valid image. The file you uploaded was either not an image or a corrupted image.',
    }
    image_field = None


    def __init__(self, *args, **kwargs):

        required = kwargs.get('required', False)
        widget = kwargs.pop('widget', None)
        self.image_field = kwargs.pop('label')

        # print widget == AdminFileWidget, isinstance(widget, AdminFileWidget), widget
        # print


        # if widget:
        # if widget == AdminFileWidget or isinstance(widget, AdminFileWidget):
        widget = CroppedImageInput( attrs={
            'class': 'image-croppable',
            'data-workon-croppedimage': self.image_field,
            'style': 'display:none;',
        })
        kwargs['widget'] = widget
        kwargs['label'] = ''

        super(CroppedImageField, self).__init__(*args, **kwargs)


    def to_python(self, data):
        return super(CroppedImageField, self).to_python(data)


from django.db import models
from workon.forms.tree import TreeModelChoiceField

class TreeManyToManyField(models.ManyToManyField):

    def formfield(self, **kwargs):
        defaults = {
            'form_class': TreeModelChoiceField
        }
        return super().formfield(**defaults)

class TreeForeignKey(models.ForeignKey):

    def formfield(self, **kwargs):
        # defaults = {
        #     'form_class': TreeModelChoiceField
        # }
        return super().formfield(**kwargs)


import uuid
from django.core.exceptions import ImproperlyConfigured
from django.views import generic
from django.forms import models as model_forms
from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import redirect, render
import workon.utils


__all__ = ['Save', 'ModalSave', 'SaveField']


class SaveField():
    def __init__(self, name, label=None, col=12):
        self.name = name
        self.meta = {
            'class': f'col s12 m{col}',
            'label': label or name,
        }


class Save(generic.UpdateView):

    layout_template_name = "workon/save/layout.html"
    template_name = "workon/views/save/save.html"
    result_template_name = "workon/views/list/_row.html"
    fields = None
    form_classes = ''
    modal_classes = ''

    def get_form_id(self):
        return uuid.uuid1()

    def get_form_class(self):
        print('HU')
        if self.fields is not None and self.form_class:
            raise ImproperlyConfigured("Specifying both 'fields' and 'form_class' is not permitted.")
        if self.form_class:
            return self.form_class
        else:
            if self.model is not None:
                # If a model has been explicitly provided, use it
                model = self.model
            elif hasattr(self, 'object') and self.object is not None:
                model = self.object.__class__
            else:
                model = self.get_queryset().model

            if self.fields is None:
                raise ImproperlyConfigured("Using ModelFormMixin (base class of %s) without the 'fields' attribute is prohibited." % self.__class__.__name__)

            fields = []
            for i, field in enumerate(self.fields):
                if isinstance(field, SaveField):
                    fields.append(field.name)
                elif isinstance(field, dict):
                    self.fields[i] = field = SaveField(**field)
                    fields.append(field.name)
                else:
                    fields.append(field)
            return model_forms.modelform_factory(model, fields=fields)


    def form_initialize(self, form):
        return form

    def get_form(self, *args, **kwargs):
        print('HU')
        self.form = super().get_form(*args, **kwargs)
        self.form_id = self.get_form_id()
        for name, field in self.form.fields.items():
            if hasattr(self, f'get_field_{name}'):
                self.form.fields[name] = getattr(self, f'get_field_{name}')()
           
        self.form_initialize(self.form)
        for field in self.form.fields.values():
            field.meta = {
                'class': f'col s12',
                'label': field.label,
            }
        for field in self.fields:
            if isinstance(field, SaveField):
                if self.form.fields.get(field.name):
                    self.form.fields[field.name].meta.update(field.meta)
        return self.form

    @property
    def save_and_continue(self):
        return bool(self.request.POST.get('_save_and_continue'))

    def get_object(self, *args, **kwargs):
        if hasattr(self, '_object'):
            return getattr(self, '_object')
        self.created = False
        try:
            self.object = super().get_object()
        except AttributeError:
            self.object = self.model()
            self.created = True

        setattr(self, '_object', self.object)
        return self.object

    def get_save_url(self):
        return self.request.get_full_path()

    def render_form(self, **kwargs):
        return workon.utils.render_content(self.request, self.template_name, self.get_context_data(**{
            'form': self.form
        }, **kwargs))

    def render_row(self, obj):
        return workon.utils.render_content(self.request, self.result_template_name, {
            'row': {
                'object': obj,
                'id': f'{obj}_{obj.pk}'
            }
        })

    def get_success_message(self, obj):
        return f'{obj} enregistré'

    def get_valid_json(self, obj, **kwargs):
        success_message = self.get_success_message(obj)
        json = {
            'notice': {
                'content': success_message,
                'classes': 'success'
            }
        }
        if self.save_and_continue:
            form = self.get_form()
            form.instance = obj
            json['replace'] = self.render_form(**kwargs)
        else:
            messages.success(self.request, success_message)
            json['redirect'] = self.request.META['HTTP_REFERER']

        json.update(kwargs)
        return json
        # [
        #     workon.utils.render_content(self.request, self.result_template_name, {
        #         'row': {
        #             'object': obj,
        #             'id': obj
        #         }
        #     }),
        # ]

    def render_valid(self, obj, **kwargs):
        return JsonResponse(self.get_valid_json(obj, **kwargs))

    def form_valid(self, *args, **kwargs):
        obj = self.form.save()
        return self.render_valid(obj)

    def render_modal_title(self):
        return str(self.form.instance)

    def render_script(self):
        return ""

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['title'] = self.render_modal_title()
        ctx['save_url'] = self.get_save_url()
        ctx['form_classes'] = self.form_classes
        ctx['modal_classes'] = self.modal_classes
        ctx['extra_script'] = self.render_script() 
        ctx['layout_template'] = self.layout_template_name
        ctx['form_id'] = self.form_id
        return ctx

class ModalSave(Save):
    template_name = "workon/views/save/_modal.html"
    
    def get_valid_json(self, obj, **kwargs):
        success_message = self.get_success_message(obj)
        json = {
            'notice': {
                'content': success_message,
                'classes': 'success'
            }
        }
        if self.save_and_continue:
            form = self.get_form()
            form.instance = obj
            json['replaceModal'] = self.render_form(**kwargs)
        else:
            messages.success(self.request, success_message)
            json['redirect'] = self.request.META['HTTP_REFERER']

        json.update(kwargs)
        return json

    def render_valid(self, obj, **kwargs):
        return JsonResponse(self.get_valid_json(obj, **kwargs))



# encoding: utf-8

import re
import os
import logging
import json

from django.conf import settings
from django import forms
from django.template import Context
from django.forms.widgets import FILE_INPUT_CONTRADICTION, CheckboxInput, FileInput
from django.template.loader import render_to_string
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe
from django.core.files.uploadedfile import InMemoryUploadedFile
try:
    from sorl.thumbnail.shortcuts import get_thumbnail
except:
    def get_thumbnail(image_url, *args, **kwargs):
        return image_url


logger = logging.getLogger(__name__)


EMBED_TYPES = {
    'youtube': [
        [
            r'(https?://)?(www\.)?'
            '(youtube|youtu|youtube-nocookie)\.(com|be)/'
            '(watch\?v=|embed/|v/|.+\?v=)?([^&=%\?\s"]{11})',

            '<iframe class="contrib-media" src="'
            'https://www.youtube.com/embed/\\6?controls=0&amp;showinfo=0"'
            'scrolling="no" frameborder="no" allowfullscreen></iframe>'
        ]
    ],
    'soundcloud': [
        [
            r'(http[s]?\:\/\/w\.soundcloud\.com\/player\/\?url=([^"]+))',
            '<iframe class="contrib-media" src="https://w.soundcloud.com/player/?url=\\2" scrolling="no" frameborder="no" allowfullscreen></iframe>'
        ],
        [
            r'(http[s]?\:\/\/soundcloud\.com\/[\d\w\-_]+/[\d\w\-_]+)',
            '<iframe class="contrib-media" src="https://w.soundcloud.com/player/?url=\\1" scrolling="no" frameborder="no" allowfullscreen></iframe>'
        ]
    ],
    'vimeo': [
        [
            r'(http[s]?\:\/\/(player\.)?vimeo\.com\/([\/\w]+)\/([\d]+))',
            """<iframe class="contrib-media" src="https://player.vimeo.com/video/\\4" scrolling="no" frameborder="no" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>"""
        ]
    ]
}


class MediaInput(forms.widgets.ClearableFileInput):
    template_name = 'contrib/forms/_media_input.html'
    change_message = "Changer"
    empty_message = "Changer"
    class Media:
        css = {
            'all': (
                'contrib/forms/media.css',
            )
        }
        js = (
            'contrib/packages/media.js',
            'contrib/forms/media.js',
        )


    def __init__(self, *args, **kwargs):
        self.authorized_types = []
        if kwargs.get('authorized_types'):
            self.authorized_types = kwargs.pop('authorized_types')
        super(MediaInput, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        # print "MediaInput.render : ", name, type(value), value, self.is_initial(value)
        if value is None:
            value = ''

        field = super(MediaInput, self).render(name, value, attrs=attrs)

        in_memory = False


        if isinstance(value, InMemoryUploadedFile):
            in_memory = True

        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        checkbox_name = self.clear_checkbox_name(name)
        checkbox_id = self.clear_checkbox_id(checkbox_name)
        context = {
            'only_file': len(self.authorized_types) == 1 and 'image' in self.authorized_types,
            'required': self.is_required,
            'field': field,
            'name': name,
            'value': value,
            'id': final_attrs['id'],
            'final_attrs': flatatt(final_attrs),
            'change_message': self.change_message,
            'empty_message': self.empty_message,
            'embed_types': json.dumps(EMBED_TYPES)
        }
        if not self.is_required:
            context['clear'] = CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
        return render_to_string(self.template_name, Context(context))


    # def decompress(self, value):
    #     if value:
    #         return value.split(' ')
    #     return [None, None]

    def value_from_datadict(self, data, files, name):
        # print 'MediaInput.value_from_datadict : ', name, (files.get(name, None), data.get(name, None))
        if not self.is_required and CheckboxInput().value_from_datadict(
                data, files, self.clear_checkbox_name(name)):
            return False


        if files:
            return files.get(name, None)
        else:
            return data.get(name, None)

        # return files.get(name, None), data.get(name, None)


class MediaField(forms.FileField):
    # widget = MediaInput
    default_error_messages = {
        'invalid_image': "Upload a valid image. The file you uploaded was either not an image or a corrupted image.",
    }

    def __init__(self, *args, **kwargs):

        # required = kwargs.get('required', False)
        #widget = kwargs.pop('widget', None)
        # widget = MediaInput( attrs={
        #     'class': 'contrib-media_field',
        #     # 'data-contrib-media_field': self.image_field,
        #     # 'style': 'display:none;',
        # })
        # kwargs['widget'] = widget
        # kwargs['label'] = ""

        super(MediaField, self).__init__(*args, **kwargs)


    def to_python(self, data):
        # print 'MediaFormField.to_python : ', data, type(data)
        return data

    def has_changed(self, initial, data):
        # print 'MediaFormField.has_changed:', data, type(data)
        if data is None:
            return super(MediaField, self).has_changed(initial, data)
        return True

    def bound_data(self, data, initial):
        # print 'MediaFormField.bound_data:', data, type(data), initial, type(initial)
        if data in [False, None] and initial:
            return initial
        return data

    def clean(self, data, initial=None):
        if data is False:
            if not self.required:
                return False
            data = None
        if not data and initial:
            return initial
        return super(MediaField, self).clean(data)


from django import forms
from django.contrib.postgres.fields import ArrayField
from django.forms import SelectMultiple
 

__all__ = ['ArrayChoiceField']


class ArraySelectMultiple(SelectMultiple):

    def value_omitted_from_data(self, data, files, name):
        return False

 
class ArrayChoiceField(ArrayField):
    """
    A field that allows us to store an array of choices.
     
    Uses Django 1.9's postgres ArrayField
    and a MultipleChoiceField for its formfield.
    """
 
    def formfield(self, **kwargs):
        defaults = {
            'form_class': forms.TypedMultipleChoiceField,
            'choices': self.base_field.choices,
            'coerce': self.base_field.to_python,
            'widget': ArraySelectMultiple
        }
        defaults.update(kwargs)
        # Skip our parent's formfield implementation completely as we don't
        # care for it.
        # pylint:disable=bad-super-call
        return super(ArrayField, self).formfield(**defaults)



# from tinymce.models import HTMLField as TinyMceHTMLField

from django.db import models
from workon.forms import HtmlInput


# class HTMLField(TinyMceHTMLField):
#     pass

class HTMLField(models.TextField):
    """
    A large string field for HTML content. It uses the TinyMCE widget in
    forms.
    """
    def __init__(self, *args, **kwargs):

        self.tinymce = kwargs.pop('tinymce', None)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):

        defaults = { 'widget': HtmlInput }
        defaults.update(kwargs)

        # As an ugly hack, we override the admin widget
        # if defaults['widget'] == admin_widgets.AdminTextareaWidget:
        #     defaults['widget'] = tinymce_widgets.AdminTinyMCE

        defaults['widget'] = HtmlInput(attrs={'placeholder': self.verbose_name}, tinymce=self.tinymce)

        return super().formfield(**defaults)

class HtmlField(HTMLField):
    pass



from django.db import models
from django import forms
from django.utils.text import capfirst
from workon.forms import ColorField as ColorFormField, ColorInput

__all__ = ['ColorField']


class ColorField(models.CharField):
    """
    A text field made to accept hexadecimal color value (#FFFFFF)
    with a color picker widget.
    """
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 7
        super(ColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorInput
        return super().formfield(**kwargs)




# # encoding: utf-8

# from django import forms
# from django.utils.translation import ugettext_lazy as _

# from localflavor.fr.forms import FRPhoneNumberField as OriginalFRPhoneNumberField

# class FRPhoneNumberField(OriginalFRPhoneNumberField):
#     default_error_messages = {
#         'invalid': _(u"Le numéro de téléphone doit correspondre au format 0X XX XX XX XX."),
#     }


import datetime, re
from urllib.parse import urlparse
from premailer import transform
from email.utils import parseaddr
from django.conf import settings
from django.utils import six
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.template import Context, RequestContext
from django.utils.html import strip_tags
from django.core.mail import get_connection, EmailMultiAlternatives

from django.core.exceptions import ImproperlyConfigured
from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import sanitize_address, DEFAULT_ATTACHMENT_MIME_TYPE
from django.core.mail.backends.smtp import EmailBackend

_email_regex = re.compile("([a-z0-9!#$%&'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+\/=?^_`"
                    "{|}~-]+)*(@|\sat\s)(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?(\.|"
                    "\sdot\s))+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?)")

__all__ = [
    'EmailStagingBackend', 
    'EmailProductionBackend',

    "extract_emails",
    "emails_to_html",
    "extract_emails_to_html",
    "is_valid_email",
    "ContentEmail",
    "HtmlTemplateEmail",
    "send_email",
    "send_mass_email",
    "send_html_email",
    "send_template_email",
    "set_mailchimp_vars",
    "clean_html_for_email",
    "make_email",
    "send_emails",
]


class EmailStagingBackend(EmailBackend):

    def route_recipients(self, recipients):
        return [
            email for name, email in settings.ADMINS
        ]

    def _send(self, message):
        orginial_receiver = ", ".join(message.to)
        message.to = self.route_recipients(message.to)
        message.cc = self.route_recipients(message.cc)
        message.bcc = self.route_recipients(message.bcc)
        message.subject += ' <TESTFOR : %s>' % orginial_receiver
        super()._send(message)


class EmailProductionBackend(EmailBackend):

    def route_recipients(self, recipients):
        return recipients

    def _send(self, message):
        super()._send(message)




def extract_emails(text):
    if text is not None:
        return list(set((email[0] for email in _email_regex.findall(text) if not email[0].startswith('//'))))
    else:
        return []


def emails_to_html(emails, reverse=True, classname=None, divider="<br />"):
    emails = [
        u'<a href="mailto:%s" %s/>%s</a>' % (
            email,
            ('class="%s" ' % classname) if classname else "",
            email.strip('/')
        ) for email in emails
    ]
    if reverse:
        emails.reverse()
    html = divider.join(emails)
    return html

def extract_emails_to_html(text, **kwargs):
    return emails_to_html(extract_emails(text), **kwargs)

def is_valid_email(email):
    if email is None:
        return None
    result = parseaddr(email.strip().lower())
    if '@' in result[1]:
        return result[1]
    else:
        return None

class ContentEmail(EmailMultiAlternatives):
    def __init__(self, subject, content, sender, receivers, content_type=None, context={}, files=[], **kwargs):
        self.is_sent = False

        subject = " ".join(subject.splitlines())
        content = kwargs.pop('body', content)
        
        if isinstance(receivers, six.string_types):
            receivers = [receivers]

        if content_type:
            super(ContentEmail, self).__init__(subject, '', sender, receivers, **kwargs)
        else:
            super(ContentEmail, self).__init__(subject, content, sender, receivers, **kwargs)

        if content_type:
            self.attach_alternative(content, content_type)
        if files:
            for file in files:
                self.attach(*file)

class HtmlTemplateEmail(EmailMultiAlternatives):

    def __init__(self, subject, html, sender, receivers, context={}, files=[], **kwargs):
        self.is_sent = False
        self.html = html

        if isinstance(receivers, six.string_types):
            receivers = [receivers]

        subject = " ".join(subject.splitlines())
        text_template = strip_tags(html)
        if kwargs.get('clean_html') == True:
            kwargs.pop('clean_html')
            self.html = clean_html_for_email(self.html)
        super().__init__(subject, text_template, sender, receivers, **kwargs)
        self.attach_alternative(self.html, "text/html")
        if files:
            for file in files:
                self.attach(*file)

# django : send_mail(subject, message, from_email, recipient_list, fail_silently=False, auth_user=None, auth_password=None, connection=None, html_message=None)
def send_email(*args, return_email=False, **kwargs):
    fail_silently = kwargs.pop('fail_silently', False)
    message = make_email(*args, **kwargs)
    if return_email:
        message.is_sent = message.send(fail_silently=fail_silently)
        return message
    return message.send(fail_silently=fail_silently)

def make_email(subject, sender, receivers, content=None, html=None, template=None, context={}, content_type=None, files=[], **kwargs):
    if template:
        html_template = get_template(template)
        html = html_template.render(context)
        message = HtmlTemplateEmail(subject, html, sender, receivers, context, files=files, **kwargs)
    elif html:
        message = HtmlTemplateEmail(subject, html, sender, receivers, context, files=files, **kwargs)
    elif content:
        message = ContentEmail(subject, content, sender, receivers, content_type=content_type, files=files, **kwargs)

    return message

def send_emails(*emails, **kwargs):
    fail_silently = kwargs.pop('fail_silently', False)
    connection = get_connection()
    connection.open()
    connection.send_messages(emails)
    connection.close()

def send_mass_email(messages, **kwargs):
    fail_silently = kwargs.pop('fail_silently', False)
    connection = get_connection()
    connection.open()
    connection.send_messages(messages)
    connection.close()

def send_html_email(subject, sender, receivers, html='', context={}, files=[], **kwargs):
    fail_silently = kwargs.pop('fail_silently', False)
    message = HtmlTemplateEmail(subject, html, sender, receivers, context, files=files, **kwargs)
    return message.send(fail_silently=fail_silently)

def send_template_email(subject, sender, receivers, template=None, context={}, files=[], **kwargs):
    html_template = get_template(template)
    context = Context(context)
    html = html_template.render(context)
    return send_html_email(subject, sender, receivers, html=html, context=context, files=files, **kwargs)


def set_mailchimp_vars(template):
    template = template.replace('*|CURRENT_YEAR|*', str(datetime.date.today().year) )
    return template


def clean_html_for_email(html):
    return transform(html)









from django.conf import settings


__all__ = ['lazy_register']


def lazy_register(register):

	if "compressor" in settings.INSTALLED_APPS:
	    @register.tag
	    def compress(parser, token):
	        from compressor.templatetags.compress import compress
	        return compress(parser, token)
	else:
	    @register.to_end_tag
	    def compress(parsed, context, token):
	        return parsed


import os
from django import template
from django.templatetags.static import static
from django.utils.safestring import mark_safe
from django.conf import settings


__all__ = ['lazy_register']
STATIC_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../static/workon')
STATIC_JS_PATH = os.path.join(STATIC_PATH, 'js')
STATIC_JS_NAME = 'workon/js'


def lazy_register(register):


    @register.inclusion_tag('workon/assets/javascripts.html')
    def javascripts(*names, **kwargs):
        global_async = kwargs.get('async', False)
        externals = ''
        internals = ''
        paths = [
        ]
        for name in names:
            if name == 'workon':                
                paths += [
                    f'{STATIC_JS_NAME}/vendors/jquery.js',
                    f'{STATIC_JS_NAME}/vendors/jquery-migrate.js',
                    f'{STATIC_JS_NAME}/vendors/jquery-ui.js',
                    f'{STATIC_JS_NAME}/vendors/moment.min.js',
                ]

                for path in os.listdir(os.path.join(STATIC_JS_PATH, name)):
                    if path.endswith('.js'):
                        paths.append(os.path.join(f'{STATIC_JS_NAME}/{name}', path))

                # paths += [

                #     f'{STATIC_JS_NAME}/plugins/tinymce/tinymce.min.js',
                # ]
                
            elif name == 'slick':               
                paths.append(f'{STATIC_JS_NAME}/plugins/slick.js')

            elif name == 'json':               
                paths.append(f'{STATIC_JS_NAME}/plugins/json.js')

            elif name == 'code':               
                paths.append(f'{STATIC_JS_NAME}/plugins/code.js')

            else:
                paths.append(name)

        for path in paths:
            if path.startswith('http') or path.startswith('//'):
                externals += f'<script type="text/javascript" src="{path}"></script>'
            else:
                internals += f'<script type="text/javascript" src="{static(path)}"></script>'

        return {
            'externals': mark_safe(externals),
            'internals': mark_safe(internals),
        }

    PACKAGES_CSS = {
        'workon': 'workon/css/workon.css',
        'materialize-icons': [
            'https://fonts.googleapis.com/icon?family=Material+Icons',
        ],
    }
    @register.inclusion_tag('workon/assets/stylesheets.html')
    def stylesheets(*names):
        internals = ''
        externals = ''
        if not names:
            names = PACKAGES_CSS.keys()
        packages = []
        for name in names:
            paths = PACKAGES_CSS.get(name)
            if paths:
                if isinstance(paths, list):
                    packages += paths
                else:
                    packages.append(paths)
            elif name == "slick":
                packages.append(name)
            else:
                packages.append(name)
        for path in packages:
            if path.startswith('http') or path.startswith('//'):
                externals += f'<link type="text/css" rel="stylesheet"  href="{path}" media="screen,projection" />'
            else:
                internals += f'<link type="text/css" rel="stylesheet"  href="{static(path)}" media="screen,projection" />'
        return {
            'externals': mark_safe(externals),
            'internals': mark_safe(internals)
        }

    @register.inclusion_tag('workon/assets/messages.html')
    def messages(messages):
        return {
            'messages': messages
        }



import sys
from django.conf.urls import url, re_path
from django.contrib.auth.decorators import login_required as login_required_statement, user_passes_test
from django.http.request import HttpRequest
from django.urls import reverse, exceptions

staff_required_statement = user_passes_test(lambda u: u.is_staff)
superuser_required_statement = user_passes_test(lambda u: u.is_superuser)

__all__ = ['route']

_previous_route_url = None


def route(  
    pattern,
    name=None,
    attach=HttpRequest,
    attach_attr=None,
    view=None,
    view_kwargs={},
    args=(),
    kwargs={},
    fail_as_previous=False,
    login_required=False,
    staff_required=False,
    superuser_required=False,
    custom_required=None
):
    global _previous_route_url

    caller_filename = sys._getframe(1).f_code.co_filename
    containers = [caller_filename]

    splitted = caller_filename.rsplit('/', 1)
    if len(splitted) > 1:
        caller_filename = f'{splitted[0]}/__init__.py'
        containers.append(caller_filename)
        
    # splitted = caller_filename.rsplit('/views/')
    # if len(splitted) > 1:
    #     caller_filename = f'{splitted[0]}/views/__init__.py'


    pattern = pattern
    # if not pattern.endswith('$'):
    #     pattern = f'{pattern}$'
    if name:
        url_name  = name.split(':')[-1]
    else:
        url_name = None
    modules = []
    for m in sys.modules.values():
        if m and '__file__' in m.__dict__:
            for caller_filename in containers:
                if m.__file__.startswith(caller_filename):
                    modules.append(m)
            # break

    if name and attach:
        def reversor(attached):
            future_args = ( method(attached) for method in args )
            future_kwargs = { attr: method(attached) for attr, method in kwargs.items() }
            if hasattr(attached, f'{attach_attr}_url_previous'):
                try:
                    return reverse(name, args=future_args, kwargs=future_kwargs)
                except exceptions.NoReverseMatch:
                    previous = getattr(attached, f'{attach_attr}_url_previous')
                    future_args = ( method(attached) for method in previous[1] )
                    future_kwargs = { attr: method(attached) for attr, method in previous[2].items() }
                    # query = previous[3]
                    return reverse(previous[0], args=future_args, kwargs=future_kwargs)
            else:
                return reverse(name, args=future_args, kwargs=future_kwargs)

        attach_attr = name.replace('-', '_').replace(':', '_') if not attach_attr else attach_attr
        setattr(attach, f'{attach_attr}_url' , reversor)
        if fail_as_previous:
            setattr(attach, f'{attach_attr}_url_previous' , _previous_route_url)

        _previous_route_url = (name, args, kwargs)

    def _wrapper(class_or_method):

        if modules:
            for module in modules:
                if 'urlpatterns' not in module.__dict__:
                    module.urlpatterns = []

            if hasattr(class_or_method, 'as_view'):
                view = class_or_method.as_view()
            else:
                view = class_or_method

            if login_required:
                view = login_required_statement(view)
            if staff_required:
                view = staff_required_statement(view)
            if superuser_required:
                view = superuser_required_statement(view)
            if custom_required:
                view = user_passes_test(custom_required)(view)

            for module in modules:
                module.urlpatterns += [ re_path(pattern, view, name=url_name ) ]

            # print('PATTERNS', module, module.__dict__.get('urlpatterns'))
            # print('\n')

        return class_or_method

    return _wrapper



# encoding: utf-8

PRIVATE_IPS_PREFIX = ('10.', '172.', '192.', )

__all__ = ["get_ip_address_from_request", "get_ip_address"]

def get_ip_address_from_request(request):
    """get the client ip from the request
    """
    remote_address = request.META.get('REMOTE_ADDR')
    # set the default value of the ip to be the REMOTE_ADDR if available
    # else None
    ip = remote_address
    # try to get the first non-proxy ip (not a private ip) from the
    # HTTP_X_FORWARDED_FOR
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        proxies = x_forwarded_for.split(',')
        # remove the private ips from the beginning
        while (len(proxies) > 0 and
                proxies[0].startswith(PRIVATE_IPS_PREFIX)):
            proxies.pop(0)
        # take the first ip which is not a private one (of a proxy)
        if len(proxies) > 0:
            ip = proxies[0]

    return ip

get_ip_address = get_ip_address_from_request


import re
import os
import logging
import locale
import json
import datetime, time


from django import forms
from django.conf import settings
from django.db.models import CharField
from django.core.exceptions import ValidationError
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe

logger = logging.getLogger(__name__)

class BusinessHoursField(forms.CharField):

    def __init__(self, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = BusinessHoursInput(
                expanded=kwargs.pop('expanded', False),
            )
        super().__init__(*args, **kwargs)



class BusinessHoursInput(forms.widgets.TextInput):

    class Media:
        css = {
            'all': (settings.STATIC_URL + 'contrib/vendors/pretty-json/pretty-json.css',)
        }
        js = (settings.STATIC_URL + 'contrib/packages/json.js',)


    def __init__(self, *args, **kwargs):
        self.expanded = kwargs.pop('expanded', False)
        super().__init__(*args, **kwargs)

    def render(self, name, value, attrs={}):
        if 'id' not in attrs:
            attrs['id'] = "id_%s" % name

        obj = json.dumps(value, ensure_ascii=False)


        return '''<div id="%(id)s" ></div><script type="text/javascript">

                    var node = new PrettyJSON.view.Node({
                        el: $('#%(id)s'),
                        data: JSON.parse(%(obj)s),
                    });
                    %(expandAll)s
                </script>
                ''' % {
                    'id' : attrs['id'],
                    'obj': obj,
                    'expandAll': 'node.expandAll();' if self.expanded else '',
                 }



from django.db import models
from workon.forms import CodeInput


__all__ = ['CodeField']


class CodeField(models.TextField):
    """
    A large string field for HTML content. It uses the TinyMCE widget in
    forms.
    """
    def __init__(self, *args, **kwargs):
        self.mode = kwargs.pop('mode', 'text')
        super(CodeField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = { 'widget': CodeInput }
        defaults.update(kwargs)


        # As an ugly hack, we override the admin widget
        # if defaults['widget'] == admin_widgets.AdminTextareaWidget:
        #     defaults['widget'] = tinymce_widgets.AdminTinyMCE

        defaults['widget'] = CodeInput(attrs={
            'placeholder': self.verbose_name,
            'mode': self.mode
        })

        return super().formfield(**defaults)



from django.shortcuts import render
from django.template.loader import get_template


__all__ = ['render_content', 'sanitize', 'html2text']


def render_content(request, template, context):
    if request:
        return str(render(request, template, context).content, 'utf-8')
    else:
        html_template = get_template(template)
        return html_template.render(context)

def sanitize(html):
    if not has_bleach:
        logger.warning('Bleach is missing for sanitizing HTML.')
        return html
    else:
        return bleach.clean(html)

def html2text(html):
    from bs4 import BeautifulSoup
    soup = BeautifulSoup(html, "lxml")
    return soup.get_text()

# def strip_tags(value):
#     "Returns the given HTML with all tags stripped"
#     return re.sub(r'<[^>]*?>', '', value)

# def strip_entities(value):
#     "Returns the given HTML with all entities (&something;) stripped"
#     return re.sub(r'&(?:\w+|#\d);', '', value)


from django import forms

class TreeModelChoiceIterator(forms.models.ModelChoiceIterator):

    def choice(self, obj):
        parent_id = getattr(obj, 'parent_id', None)
        level = getattr(obj, getattr(self.queryset.model._meta, 'level_attr', 'level'), 1)
        return super().choice(obj) + (parent_id, level)

class TreeSelect(forms.SelectMultiple):

    template_name = "workon/forms/widgets/_treeselect.html"

    def get_context(self, name, value, attrs):
        by_ids = {}
        test_selected = lambda o: None
        if value:
            if isinstance(value, list):
                test_selected = lambda o: o in value
            else:
                test_selected = lambda o: o == value

        for choice in self.choices:
            by_ids[choice[0]] = {
                'items': [],
                'is_selected': test_selected(choice[0]),
                'label': choice[1],
                'parent_id': choice[2],
                'level': choice[3],
                'value': choice[0]
            }
        tree = {}
        for pk, item in by_ids.items():
            if item['parent_id']:
                by_ids[item['parent_id']]['items'].append(item)
                if item['is_selected']:
                    by_ids[item['parent_id']]['is_selected'] = True

            else:
                tree[pk] = item

        context = {
            'tree': tree.values(),
            'is_hidden': self.is_hidden,
            'name': name,
            'required': self.is_required,
            'value': self.format_value(value),
            'attrs': self.build_attrs(self.attrs, attrs),
            'template_name': self.template_name,
        }
        return context


class TreeModelChoiceField(forms.ModelMultipleChoiceField):
    widget = TreeSelect
    iterator = TreeModelChoiceIterator



import uuid
from django.core.exceptions import ImproperlyConfigured
from django.views import generic
from django.forms import models as model_forms
from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import redirect, render
import workon.utils


__all__ = ['View', 'ModalView']


class View(generic.DetailView):
	pass


class ModalView(generic.DetailView):
	pass



import json
import re
import os
from urllib.parse import urlparse, urlencode, parse_qs, urlsplit, urlunsplit
from django.conf import settings
from django import template, forms
from django.templatetags.static import static
from django.template.loader import get_template
from django.utils.safestring import mark_safe
from django.templatetags.static import StaticNode, static as original_static, do_static as original_do_static
from django.core.files.storage import get_storage_class, FileSystemStorage
from django import forms
import workon.utils


__all__ = ['lazy_register']


def lazy_register(register):
    
    @register.filter
    def absolute_url(url): return workon.utils.canonical_url(url)

    @register.filter
    def static(url): return original_static(url)

    @register.filter
    def external_url(url): return workon.utils.append_protocol(url)

    @register.filter
    def absolute_static(url): return absolute_url(original_static(url))

    class AbsoluteStaticNode(StaticNode):
        def url(self, context):
            path = self.path.resolve(context)
            return absolute_url(self.handle_simple(path))

    @register.tag('absolute_static')
    def absolute_static_tag(parser, token): 
        return AbsoluteStaticNode.handle_token(parser, token)

    @register.tag('static')
    def do_static(parser, token):
        return StaticNode.handle_token(parser, token)

    @register.filter
    def static_image(url):
        storage_class = get_storage_class(settings.STATICFILES_STORAGE)
        storage = storage_class()
        image = ImageFile(storage.open(url))
        image.storage = storage
        return image, image.url

    @register.filter
    def replace_urls_to_href(text):
        return mark_safe(workon.utils.replace_urls_to_href(text))

    @register.simple_tag()
    def url_replace_param(url, param_name, param_value):

        scheme, netloc, path, query_string, fragment = urlsplit(url)
        query_params = parse_qs(query_string)

        query_params[param_name] = [param_value]
        new_query_string = urlencode(query_params, doseq=True)

        return mark_safe(urlunsplit((scheme, netloc, path, new_query_string, fragment)))



import os
default_app_config = 'workon.conf.WorkonConfig'

if os.environ.get('DJANGO_SETTINGS_MODULE'):

	from workon.utils import *
	from workon.views import *
	from workon.fields import *

	# from workon.models.tracked import (
	# 	Tracked as TrackedModel
	# )


from django.views import generic
from django.http import JsonResponse
from django.contrib import messages


__all__ = ['Delete', 'ModalDelete']


class Delete(generic.DeleteView):

    def get_success_message(self, obj):
        return f'{obj} supprimé'

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        pk = self.object.pk
        self.object.delete()

        success_message = self.get_success_message(self.object)
        json = {
            'notice': {
                'content': success_message,
                'classes': 'success'
            },
        }

        messages.success(self.request, success_message)
        json['redirect'] = self.request.META['HTTP_REFERER']

        # return JsonResponse({
        #     'notice': {
        #         'content': f'{self.object} supprimé',
        #         'classes': 'success'
        #     },
        #     'remove': f'{self.object}_{self.object.pk}'
        # })

        return JsonResponse(json)


class ModalDelete(Delete):    
    template_name = "workon/views/delete/_modal.html"


from django.db import models
from django import forms
from django.utils.text import capfirst
from workon.forms import PriceField as PriceFormField, PriceInput


class PriceField(models.DecimalField):
    """
    A text field made to accept hexadecimal color value (#FFFFFF)
    with a color picker widget.
    """
    def __init__(self, *args, **kwargs):
        kwargs['decimal_places'] = kwargs.get('decimal_places', 2)
        kwargs['max_digits'] = kwargs.get('max_digits', 21)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['form_class'] = PriceFormField
        return super().formfield(**kwargs)



from .color import ColorField, ColorInput, RGBColorField
from .icon import IconField, IconInput
from .price import PriceField, PriceInput
from .image import ImageInput, ImageField, CroppedImageField, CroppedImageInput
from .date import DateField, DateTimeField, TimeField, DateTimeInput, DateInput, TimeInput
from .daterange import DateRangeField, DateRangeInput
from .media import MediaInput, MediaField
# from .advanced_media import AdvancedMediaInput, AdvancedMediaField
from .html import HtmlField, HtmlInput
from .text import TextField, TextInput
from .tree import TreeModelChoiceField, TreeSelect
# from .content_type import GenericContentTypeSelect
from .json_py import JSONField, JSONReadOnlyInput
from .info import InfoField
from .code import CodeField, CodeInput
from .embed import EmbedField, EmbedInput

from .select2 import *
# from workon.contrib.select2.forms import *
# from ..contrib.auth.forms import *


from workon.fields.array import *
from workon.fields.code import *
from workon.fields.color import *
from workon.fields.icon import IconField
from workon.fields.price import PriceField
from workon.fields.image import *
from workon.fields.percent import PercentField
from workon.fields.file import ContentTypeRestrictedFileField, UniqueFilename, unique_filename
from workon.fields.html import HtmlField, HTMLField
from workon.fields.date import DateTimeField, DateField, TimeField
from workon.fields.tree import TreeManyToManyField, TreeForeignKey
from workon.fields.embed import EmbedField
from django.contrib.postgres.fields import JSONField


from django.db import models
from workon.forms import EmbedInput
from workon.forms.embed import EMBED_TYPES
# - Youtube
# - Soundcloud
# - Vimeo
# - Bandcamp
# - Pinterest
# - Mixcloud
# - Dailymotion


class EmbedDescriptor(object):

    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            raise AttributeError(
                "The '%s' attribute can only be accessed from %s instances."
                % (self.field.name, owner.__name__))


        value = instance.__dict__[self.field.name]

        if value in [None, False, '']:
            value = None

        elif isinstance(value, six.string_types):

            for type_name, patterns in EMBED_TYPES.items():
                for pattern in patterns:
                    # regex = re.compile(pattern)
                    result = re.search(pattern[0], type_value[0])
                    if result:
                        is_embed = True
                        value_type = type_name
                        value = MediaFieldEmbed(instance, self.field, result.group(0), type=value_type)

                    else:
                        value_type = 'youtube'
                        value = MediaFieldEmbed(instance, self.field, type_value[0], type=value_type)
                # value_type = 'youtube'
                # value = MediaFieldEmbed(instance, self.field, type_value[0], type=value_type)

            else:
                value_type = type_value[0]
                if value_type in ['image']:
                    value = MediaFieldImage(instance, self.field, type_value[1])
                else:
                    value = MediaFieldEmbed(instance, self.field, type_value[1], type=value_type)


            # value = 'image|%s' % name
            # attr = self.field.attr_class(instance, self.field, value)
            # instance.__dict__[self.field.name] = attr

        # Other types of files may be assigned as well, but they need to have
        # the FieldFile interface added to them. Thus, we wrap any other type of
        # File inside a FieldFile (well, the field's attr_class, which is
        # usually FieldFile).
        elif (isinstance(value, File)) and \
            not isinstance(value, MediaFieldImage):
            value = MediaFieldImage(instance, self.field, value.name)
            value.file = instance.__dict__[self.field.name]
            value._committed = False

        # Finally, because of the (some would say boneheaded) way pickle works,
        # the underlying FieldFile might not actually itself have an associated
        # file. So we need to reset the details of the FieldFile in those cases.
        elif isinstance(value, MediaFieldImage): # and not hasattr(value, 'field'):
            value.instance = instance
            value.field = self.field
            value.storage = self.field.storage
            # print "Already an image", value.name

        # Finally, because of the (some would say boneheaded) way pickle works,
        # the underlying FieldFile might not actually itself have an associated
        # file. So we need to reset the details of the FieldFile in those cases.
        elif isinstance(value, MediaFieldEmbed): # and not hasattr(value, 'field'):
            value.instance = instance
            value.field = self.field

        else:
            value = None

        instance.__dict__[self.field.name] = value
        return instance.__dict__[self.field.name]

    def __set__(self, instance, value):
        instance.__dict__[self.field.name] = value


class EmbedField(models.TextField):
    descriptor_class = EmbedDescriptor

    def __init__(self, *args, **kwargs):

        self.authorized_types = ['image', 'youtube', 'dailymotion']
        if kwargs.get('authorized_types'):
            self.authorized_types = kwargs.pop('authorized_types')

        kwargs['max_length'] = 255

        super(EmbedField, self).__init__(*args, **kwargs)


    def formfield(self, **kwargs):
        defaults = { 'widget': EmbedInput }
        defaults.update(kwargs)
        # defaults['form_class'] = EmbedField
        defaults['widget'] = EmbedInput(
            attrs = { 'class': 'image-ratio', },
            authorized_types = self.authorized_types,
        )
        return super(EmbedField, self).formfield(**defaults)

